import 'package:flutter/material.dart';
import 'package:sp_webview/sp_webview.dart';

void main() {
  runApp(
    MaterialApp(
      theme: ThemeData(useMaterial3: true),
      home: const WebViewApp(),
    ),
  );
}

class WebViewApp extends StatefulWidget {
  const WebViewApp({super.key});

  @override
  State<WebViewApp> createState() => _WebViewAppState();
}

class _WebViewAppState extends State<WebViewApp> {
  var textt = 'Test';
  var loadSceneByID;
  var sayAudio;
  var sayText;
  final ValueNotifier<String> callbackMessages = ValueNotifier<String>('');
  late final ScrollController _scrollController;
  
  // SITEPAL EMBED CODE - START
  // Replace below below placeholder values XXXXX, YYYYY, ZZZZZ with the corresponding values for your own scene.
  // For example, if your SitePal Scene's embed code looks like this (these are placeholders & not actual values):
  //        
  //	AC_VHost_Embed(XXXXX,600,800,"",1,1,YYYYY,0,1,0,"ZZZZZ",0,0);
  //
  // Copy the entire parameter string, and update the 10th parameter from 0 to 1.
  // See more detailed instructions 
  
  final String embed = 'XXXXXXX,600,800,"",1,1,YYYYYYY,0,1,1,"ZZZZZZZZ",0,0';
  
  // SITEPAL EMBED CODE - END
  
final jsFunctions = '''
    function vh_sceneLoaded() {
      window.setStatus(0,1,0,1);
      Print.postMessage("Received 'vh_sceneLoaded' event");
    }
    function vh_audioEnded() {
      Print.postMessage("Received 'vh_audioEnded' event");
    }
    function vh_ttsLoaded() {
      Print.postMessage("Received 'vh_ttsLoaded' event");
    }
    function vh_talkStarted() {
      Print.postMessage("Received 'vh_talkStarted' event");
    }
    function vh_talkEnded() {
      Print.postMessage("Received 'vh_talkEnded' event");
    }
    function vh_audioProgress(percentPlayed) {
      Print.postMessage("Received 'vh_audioProgress' at " + percentPlayed + "%");
    }
  ''';
  void onButtonPressed(String buttonLabel) {
    setState(() {
      textt = buttonLabel;
      if (buttonLabel == '2D ILLUSTRATED') {
        loadSceneByID = 2722664;
        sayAudio = null;
        sayText = null;
        callbackMessages.value = '';
      }
      if (buttonLabel == '3D PHOTOFACE') {
        loadSceneByID = 2695693;
        sayAudio = null;
        sayText = null;
        callbackMessages.value = '';
      }
      if (buttonLabel == 'FULL BODY') {
        loadSceneByID = 2695703;
        sayAudio = null;
        sayText = null;
        callbackMessages.value = '';
      }
      if (buttonLabel == 'sayAudio') {
        sayAudio = 'sayAudioExample';
        loadSceneByID = null;
        sayText = null;
      }
      if (buttonLabel == 'sayText') {
        sayText = ["Welcome to our text to speech API example", 3, 1, 3];
        sayAudio = '';
        loadSceneByID = null;
      }
    });
  }

  @override
  void initState() {
    super.initState();
    _scrollController = ScrollController();
    callbackMessages.addListener(_scrollToBottom);
  }

  @override
  void dispose() {
    _scrollController.dispose();
  callbackMessages.removeListener(_scrollToBottom);
    callbackMessages.dispose();
    super.dispose();
  }

  void _scrollToBottom() {
  WidgetsBinding.instance.addPostFrameCallback((_) {
    _scrollController.jumpTo(_scrollController.position.maxScrollExtent);
  });
}

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('Flutter WebView'),
      ),
      body: Container(
        padding: EdgeInsets.all(5),
        color: Color.fromARGB(1, 227, 89, 227),
        child: Column(
          children: [
              SpWebView(
                embed: embed,
                loadSceneByID: '$loadSceneByID',
                sayAudio: '$sayAudio',
                sayText: sayText,
                jsFunctions: jsFunctions,
                setCallbackOutput: (log) {
                  print('callback received:  $log');
                  callbackMessages.value = callbackMessages.value + log + '\n'; // Update the ValueNotifier
                },
              ),

            ValueListenableBuilder<String>(
              valueListenable: callbackMessages,
              builder: (context, message, child) {
                return Container(
                  height: 200,
                  margin: const EdgeInsets.symmetric(horizontal: 20, vertical: 10),
                  color: Color.fromARGB(255, 245, 235, 235),
                  child: Align(
                    alignment: Alignment.topLeft,
                    child: Padding(
                      padding: const EdgeInsets.all(10),
                      child:SingleChildScrollView(
                        controller: _scrollController,
                        child: Text(message),
                        )

                    ),
                  ),
                );
              },
            ),
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              children: [
                ElevatedButton(
                  onPressed: () => onButtonPressed('2D ILLUSTRATED'),
                  child: const Text('2D ILLUSTRATED'),
                ),
                ElevatedButton(
                  onPressed: () => onButtonPressed('3D PHOTOFACE'),
                  child: const Text('3D PHOTOFACE'),
                ),
              ],
            ),
            ElevatedButton(
              onPressed: () => onButtonPressed('FULL BODY'),
              child: const Text('FULL BODY'),
            ),
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              children: [
                ElevatedButton(
                  onPressed: () => onButtonPressed('sayAudio'),
                  child: const Text('sayAudio()'),
                ),
                ElevatedButton(
                  onPressed: () => onButtonPressed('sayText'),
                  child: const Text('sayText()'),
                ),
              ],
            ),
          ],
        ),
      ),
    );
  }
}